BOOTLOADER_ADAFRUIT = "adafruit"
BOOTLOADER_ADAFRUIT_NRF52_SD132 = "adafruit_nrf52_sd132"
BOOTLOADER_ADAFRUIT_NRF52_SD140_V6 = "adafruit_nrf52_sd140_v6"
BOOTLOADER_ADAFRUIT_NRF52_SD140_V7 = "adafruit_nrf52_sd140_v7"

EXTRA_ADC = [
    "VDD",
    "VDDHDIV5",
]
AIN_TO_GPIO = {
    "AIN0": 2,
    "AIN1": 3,
    "AIN2": 4,
    "AIN3": 5,
    "AIN4": 28,
    "AIN5": 29,
    "AIN6": 30,
    "AIN7": 31,
}
