#pragma once

namespace esphome {

/*! @mainpage ESPHome source code documentation
 This documentation provides references to the ESPHome source code classes and methods.

  @details This documentation site is purely for reference and does not contain any user documentation.
  If you are contributing to ESPHome or building an ESPHome component, then you should be starting at
  https://developers.esphome.io.
*/

}  // namespace esphome
