OPENTHERM = "opentherm"

CONF_OPENTHERM_ID = "opentherm_id"
CONF_DATA_TYPE = "data_type"

SENSOR = "sensor"
BINARY_SENSOR = "binary_sensor"
SWITCH = "switch"
NUMBER = "number"
OUTPUT = "output"
INPUT_SENSOR = "input_sensor"
SETTING = "setting"
